package shef.dcs.cashews.parts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.tools.DirectEditManager;
import org.eclipse.jface.viewers.TextCellEditor;

import shef.dcs.cashews.figures.NodeFigure;
import shef.dcs.cashews.model.Node;
import shef.dcs.cashews.policies.NodeDirectEditPolicy;


public class NodePart extends AbstractGraphicalEditPart implements PropertyChangeListener {

   protected DirectEditManager manager;


   //------------------------------------------------------------------------
   // Overridden from AbstractGraphicalEditPart

   // override activate to register with the model for property changes
   public void activate() {
      if (isActive()) {
         return;
      }
      super.activate();
      ((Node) getModel()).addPropertyChangeListener(this);
   }

   // override deactivate to deregister with the model
   public void deactivate() {
      if (!isActive()) {
         return;
      }
      super.deactivate();
      ((Node) getModel()).removePropertyChangeListener(this);
   }


   //------------------------------------------------------------------------
   // Overridden from AbstractEditPart

   protected void refreshVisuals() {
      Node node = (Node) this.getModel();
      Point loc = node.getLocation();
      Dimension size = new Dimension(150, 40);
      Rectangle rectangle = new Rectangle(loc, size);

      ((NodeFigure) this.getFigure()).setName(((Node) this.getModel()).getName());

      // tells the parent part (in this case DiagramPart) that this part
      // and its figure are to be constrained to the given rectangle
       ((GraphicalEditPart) getParent()).setLayoutConstraint(this, getFigure(), rectangle);
   }

   public void performRequest(Request request) {
      if (request.getType() == RequestConstants.REQ_DIRECT_EDIT) {
         if (manager == null) {
            NodeFigure nodeFigure = (NodeFigure) getFigure();
            manager = new NodeDirectEditManager(this, TextCellEditor.class, new NodeCellEditorLocator(nodeFigure), nodeFigure);
         }
         manager.show();
      }
   }


   //------------------------------------------------------------------------
   // Abstract methods from AbstractGraphicalEditPart

   protected IFigure createFigure() {
      return new NodeFigure();
   }


   //------------------------------------------------------------------------
   // Abstract methods from AbstractEditPart

   protected void createEditPolicies() {
      installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new NodeDirectEditPolicy());
   }


   //------------------------------------------------------------------------
   // Abstract methods from PropertyChangeListener

   // catch changes to LOCATION or NAME and refresh visuals for this part
   public void propertyChange(PropertyChangeEvent changeEvent) {
      if (changeEvent.getPropertyName().equals(Node.LOCATION)) {
         refreshVisuals();
      }
      else if (changeEvent.getPropertyName().equals(Node.NAME)) {
         refreshVisuals();
      }
   }
}